
#include <log4cplus/logger.h>
#include <log4cplus/configurator.h>
#include <log4cplus/ndc.h>
#include <log4cplus/helpers/loglog.h>
#include <log4cplus/helpers/stringhelper.h>
#include <log4cplus/helpers/sleep.h>
#include <log4cplus/loggingmacros.h>
#include <log4cplus/thread/threads.h>
#include <log4cplus/tracelogger.h>
#include <log4cplus/streams.h>
#include <exception>
#include <iostream>
#include <string>

#include <cstdlib>
#include <ctime>
#include <sys/time.h>

using namespace std;
using namespace log4cplus;
using namespace log4cplus::helpers;
using namespace log4cplus::thread;

tstring availableLoggers[] =
{
    LOG4CPLUS_TEXT("test.log_1"),
    LOG4CPLUS_TEXT("test.log_2"),
    LOG4CPLUS_TEXT("test.log_3"),
    LOG4CPLUS_TEXT("test.log_4"),
    LOG4CPLUS_TEXT("test.log_5"),
    LOG4CPLUS_TEXT("test.log_6"),
    LOG4CPLUS_TEXT("test.log_7"),
    LOG4CPLUS_TEXT("test.log_8"),
    LOG4CPLUS_TEXT("test.log_9")
};

#define lengthof(x) (sizeof(x)/sizeof(x[0]))

#define MILLIS_TO_NANOS 1000
#define NUM_THREADS 40
#define NUM_LOOPS 50000

class TestThread
    : public AbstractThread
{
public:
    TestThread(tstring const &n) 
        : name(n)
        , mylogger(Logger::getInstance(LOG4CPLUS_TEXT("test.TestThread")))
    {
    }

    virtual void run()
    {
        try
        {
            LOG4CPLUS_WARN(mylogger, name + LOG4CPLUS_TEXT(" TestThread.run()- Starting..."));
            NDC& ndc = getNDC();
            NDCContextCreator _first_ndc(name);
            LOG4CPLUS_DEBUG(mylogger, "Entering Run()...");
            for(int i=0; i<NUM_LOOPS; ++i)
            {
                tostringstream s;
                s << "loop-" << i;
                NDCContextCreator _ndc(s.str());
                doSomething();
            }
            LOG4CPLUS_DEBUG(mylogger, "Exiting run()...");

            ndc.remove();
        }
        catch(exception& e)
        {
            LOG4CPLUS_FATAL(mylogger, "TestThread.run()- Exception occurred: " << e.what());
        }
        catch(...)
        {
            LOG4CPLUS_FATAL(mylogger, "TestThread.run()- Exception occurred!!");
        }
        LOG4CPLUS_WARN(mylogger, name << " TestThread.run()- Finished");
    } // end "run"

    void doSomething()
    {
        LOG4CPLUS_TRACE_METHOD(mylogger, LOG4CPLUS_TEXT("doSomething()"));

        int n = 1 + ( rand() % ( lengthof(availableLoggers) - 1 ) );
        for( int i = 0; i < n; ++i )
        {
            printMsgs( availableLoggers[rand() % lengthof(availableLoggers)] );
            sleep(0, (50 + ( rand() % 50 )) * MILLIS_TO_NANOS);
        }
        log4cplus::thread::yield();
    }

    void printMsgs(tstring const &logger_name)
    {
        Logger logger = Logger::getInstance(logger_name);
        LOG4CPLUS_TRACE_METHOD(logger, LOG4CPLUS_TEXT("printMsgs()"));
        LOG4CPLUS_DEBUG(logger, "printMsgs()");
        LOG4CPLUS_INFO(logger, "printMsgs()");
        LOG4CPLUS_WARN(logger, "printMsgs()");
        LOG4CPLUS_ERROR(logger, "printMsgs()");
    }

private:
    tstring name;
    Logger mylogger;
};

static volatile int endTouch = 0;

class TouchThread
    : public AbstractThread
{
public:
    TouchThread(string const &fn, long sm) 
        : filename(fn)
        , sleep_millis(sm)
    {}

    virtual void run()
    {
        while( !endTouch ) // FIXME better done using condition ...
        {
            // XXX only Unix! (SUSv2)
            struct timeval times[2];
            gettimeofday( &times[0], NULL );
            times[1] = times[0];
            LOG4CPLUS_INFO( Logger::getInstance(LOG4CPLUS_TEXT("test.TouchThread")),
                            tstring(LOG4CPLUS_TEXT("touching ")) + filename );
            utimes(filename.c_str(), times);
            sleep(0, sleep_millis * MILLIS_TO_NANOS);
        }
    }
private:
    string filename;
    long sleep_millis;
};

int
main()
{
    tcout << LOG4CPLUS_TEXT("Entering main()...") << endl;
    // LogLog::getLogLog()->setInternalDebugging(true);
    srand(time(NULL));

    try
    {
        ConfigureAndWatchThread configureThread(
            LOG4CPLUS_TEXT("log4cplus.properties"), 5 * 1000);
        TouchThread touchThread("log4cplus.properties", 5 * 1000 );
        touchThread.start();
        //log4cplus::helpers::LogLog::getLogLog()->setInternalDebugging(true);
        Logger logger = Logger::getInstance(LOG4CPLUS_TEXT("main"));
        LogLevel ll = logger.getLogLevel();
        tcout << "main Priority: " << getLogLevelManager().toString(ll) << endl;

        log4cplus::helpers::SharedObjectPtr<TestThread> threads[NUM_THREADS];
        int i = 0;
        for(i=0; i<NUM_THREADS; ++i) {
            tostringstream s;
            s << "Thread-" << i;
            threads[i] = new TestThread(s.str());
        }

        for(i=0; i<NUM_THREADS; ++i) {
            threads[i]->start();
        }
        LOG4CPLUS_DEBUG(logger, "All Threads started...");

        for(i=0; i<NUM_THREADS; ++i) {
            while(threads[i]->isRunning()) {
                sleep(0, 200 * MILLIS_TO_NANOS);
            }
        }
        ++endTouch;
        LOG4CPLUS_INFO(logger, "End main()...");
    }
    catch(std::exception &e)
    {
        LOG4CPLUS_FATAL(Logger::getRoot(), "main()- Exception occured: " << e.what());
    }
    catch(...)
    {
        LOG4CPLUS_FATAL(Logger::getRoot(), "main()- Exception occured");
    }

    log4cplus::Logger::shutdown();

    tcout << LOG4CPLUS_TEXT("Exiting main()...") << endl;

    return 0;
}

